/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package es.rickyepoderi.compiler;

import java.util.StringTokenizer;
import org.eclipse.jdt.internal.compiler.env.ICompilationUnit;

/**
 * Class that represents a single source file to compile. It is very simple, 
 * just the name of the class to compile (fully qualified name class with the
 * package) and the char[] with the source of the file.
 * 
 * @author ricky
 */
class CompilationUnit implements ICompilationUnit {

    /**
     * Fully qualified name for the main class in the file.
     */
    private final String className;
    
    /**
     * Source of the file.
     */
    private final char[] source;

    /**
     * Constructor using arguments.
     * @param className The class name
     * @param source The source
     */
    CompilationUnit(String className, char[] source) {
        this.className = className;
        this.source = source;
    }

    /**
     * Returns the file name. The fully qualified name (ej: es.sample.Main) is
     * transformed into a file path (es/sample/Main.java).
     * @return The name (path) of the source file.
     */
    @Override
    public char[] getFileName() {
        return (className.replace(".", "/") + ".java").toCharArray();
    }

    /**
     * returns the contents, in this implementation the source array
     * is returned.
     * @return The source of the java compilation unit
     */
    @Override
    public char[] getContents() {
        return source;
    }

    /**
     * Returns the name of the main class inside the unit.
     * @return Return the class name inside the fully qualified name
     */
    @Override
    public char[] getMainTypeName() {
        int dot = className.lastIndexOf('.');
        if (dot > 0) {
            return className.substring(dot + 1).toCharArray();
        }
        return className.toCharArray();
    }

    /**
     * Return the package name (in the previous example "es.sample").
     * @return The array of package names
     */
    @Override
    public char[][] getPackageName() {
        StringTokenizer izer = new StringTokenizer(className, ".");
        char[][] result = new char[izer.countTokens() - 1][];
        for (int i = 0; i < result.length; i++) {
            String tok = izer.nextToken();
            result[i] = tok.toCharArray();
        }
        return result;
    }

    /**
     * Always false in this implementation.
     * @return false
     */
    @Override
    public boolean ignoreOptionalProblems() {
        return false;
    }
}
