/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package es.rickyepoderi.compiler;

import java.io.File;
import java.util.HashMap;
import java.util.Map;
import javax.servlet.ServletContext;
import javax.servlet.jsp.tagext.TagLibraryInfo;
import org.apache.jasper.JspC;
import org.apache.jasper.Options;
import org.apache.jasper.compiler.JspConfig;
import org.apache.jasper.compiler.TagPluginManager;
import org.apache.jasper.compiler.TldCache;
import org.eclipse.jdt.internal.compiler.impl.CompilerOptions;

/**
 * Default compiler options for compiling inside ECJ, jasper (JDT).
 * 
 * @author ricky
 */
public class DefaultCompilerOptions implements Options {

    private boolean errorOnUseBeanInvalidClassAttribute = false;
    private boolean keepGenerated = false;
    private boolean poolingEnabled = true;
    private boolean mappedFile = false;
    private boolean classDebugInfo = true;
    private int checkInterval = 0;
    private boolean development = false;
    private boolean displaySourceFragment = true;
    private boolean smapSuppressed = true;
    private boolean smapDumped = false;
    private String ieClassId = JspC.DEFAULT_IE_CLASS_ID;
    private boolean trimSpaces = false;
    private File scratchDir = null; //new File(System.getProperty("java.io.tmpdir")); // TODO: put null to check no file is created
    private String classpath = System.getProperty("java.class.path");
    private String compiler = null;
    private String compilerTargetVM = CompilerOptions.VERSION_1_7;
    private String compilerSourceVM = CompilerOptions.VERSION_1_7;
    private String compilerClassName = "es.rickyepoderi.compiler.jsp.MemoryCompiler";
    private String javaEncoding = "UTF-8";
    private boolean fork = false;
    private boolean xpoweredBy = false;
    private boolean genStringAsCharArray = false;
    private int modificationTestInterval = 0;
    private boolean recompileOnFail = false;
    private boolean caching = true;
    private final Map<String, TagLibraryInfo> cache = new HashMap<>();
    private int maxLoadedJsps = -1;
    private int jspIdleTimeout = -1;
    private TldCache tldCache = null;
    private JspConfig jspConfig = null;
    private TagPluginManager tagPluginManager = null;
    
    /**
     * Empty constructor.
     */
    public DefaultCompilerOptions() {
        // NOOP
    }
    
    /**
     * Constructor using a Servlet Context (used for JSP compiling).
     * @param ctx  The servlet context to use
     */
    public DefaultCompilerOptions(ServletContext ctx) {
        tldCache = TldCache.getInstance(ctx);
        jspConfig = new JspConfig(ctx);
        tagPluginManager = new TagPluginManager(ctx);
    }
    
    //
    // GETTERS
    //
    
    @Override
    public boolean getErrorOnUseBeanInvalidClassAttribute() {
        return errorOnUseBeanInvalidClassAttribute;
    }

    @Override
    public boolean getKeepGenerated() {
        return keepGenerated;
    }

    @Override
    public boolean isPoolingEnabled() {
        return poolingEnabled;
    }

    @Override
    public boolean getMappedFile() {
        return mappedFile;
    }

    @Override
    public boolean getClassDebugInfo() {
        return classDebugInfo;
    }

    @Override
    public int getCheckInterval() {
        return checkInterval;
    }

    @Override
    public boolean getDevelopment() {
        return development;
    }

    @Override
    public boolean getDisplaySourceFragment() {
        return displaySourceFragment;
    }

    @Override
    public boolean isSmapSuppressed() {
        return smapSuppressed;
    }

    @Override
    public boolean isSmapDumped() {
        return smapDumped;
    }

    @Override
    public boolean getTrimSpaces() {
        return trimSpaces;
    }

    @Override
    public String getIeClassId() {
        return ieClassId;
    }

    @Override
    public File getScratchDir() {
        return scratchDir;
    }

    @Override
    public String getClassPath() {
        return classpath;
    }

    @Override
    public String getCompiler() {
        return compiler;
    }

    @Override
    public String getCompilerTargetVM() {
        return compilerTargetVM;
    }

    @Override
    public String getCompilerSourceVM() {
        return compilerSourceVM;
    }

    @Override
    public String getCompilerClassName() {
        return compilerClassName;
    }

    @Override
    public TldCache getTldCache() {
        return tldCache;
    }

    @Override
    public String getJavaEncoding() {
        return javaEncoding;
    }

    @Override
    public boolean getFork() {
        return fork;
    }

    @Override
    public JspConfig getJspConfig() {
        return jspConfig;
    }

    @Override
    public boolean isXpoweredBy() {
        return xpoweredBy;
    }

    @Override
    public TagPluginManager getTagPluginManager() {
        return tagPluginManager;
    }

    @Override
    public boolean genStringAsCharArray() {
        return genStringAsCharArray;
    }

    @Override
    public int getModificationTestInterval() {
        return modificationTestInterval;
    }

    @Override
    public boolean getRecompileOnFail() {
        return recompileOnFail;
    }

    @Override
    public boolean isCaching() {
        return caching;
    }

    @Override
    public Map<String, TagLibraryInfo> getCache() {
        return cache;
    }

    @Override
    public int getMaxLoadedJsps() {
        return maxLoadedJsps;
    }

    @Override
    public int getJspIdleTimeout() {
        return jspIdleTimeout;
    }
    
    //
    // SETTERS
    //
    
    public void setErrorOnUseBeanInvalidClassAttribute(boolean errorOnUseBeanInvalidClassAttribute) {
        this.errorOnUseBeanInvalidClassAttribute = errorOnUseBeanInvalidClassAttribute;
    }

    public void setKeepGenerated(boolean keepGenerated) {
        this.keepGenerated = keepGenerated;
    }

    public void setPoolingEnabled(boolean poolingEnabled) {
        this.poolingEnabled = poolingEnabled;
    }

    public void setMappedFile(boolean mappedFile) {
        this.mappedFile = mappedFile;
    }

    public void setClassDebugInfo(boolean classDebugInfo) {
        this.classDebugInfo = classDebugInfo;
    }

    public void setCheckInterval(int checkInterval) {
        this.checkInterval = checkInterval;
    }

    public void setDevelopment(boolean development) {
        this.development = development;
    }

    public void setDisplaySourceFragment(boolean displaySourceFragment) {
        this.displaySourceFragment = displaySourceFragment;
    }

    public void setSmapSuppressed(boolean smapSuppressed) {
        this.smapSuppressed = smapSuppressed;
    }

    public void setSmapDumped(boolean smapDumped) {
        this.smapDumped = smapDumped;
    }

    public void setIeClassId(String ieClassId) {
        this.ieClassId = ieClassId;
    }

    public void setTrimSpaces(boolean trimSpaces) {
        this.trimSpaces = trimSpaces;
    }

    public void setScratchDir(File scratchDir) {
        this.scratchDir = scratchDir;
    }

    public void setClasspath(String classpath) {
        this.classpath = classpath;
    }

    public void setCompiler(String compiler) {
        this.compiler = compiler;
    }

    public void setCompilerTargetVM(String compilerTargetVM) {
        this.compilerTargetVM = compilerTargetVM;
    }

    public void setCompilerSourceVM(String compilerSourceVM) {
        this.compilerSourceVM = compilerSourceVM;
    }

    public void setCompilerClassName(String compilerClassName) {
        this.compilerClassName = compilerClassName;
    }

    public void setJavaEncoding(String javaEncoding) {
        this.javaEncoding = javaEncoding;
    }

    public void setFork(boolean fork) {
        this.fork = fork;
    }

    public void setXpoweredBy(boolean xpoweredBy) {
        this.xpoweredBy = xpoweredBy;
    }

    public void setGenStringAsCharArray(boolean genStringAsCharArray) {
        this.genStringAsCharArray = genStringAsCharArray;
    }

    public void setModificationTestInterval(int modificationTestInterval) {
        this.modificationTestInterval = modificationTestInterval;
    }

    public void setRecompileOnFail(boolean recompileOnFail) {
        this.recompileOnFail = recompileOnFail;
    }

    public void setCaching(boolean caching) {
        this.caching = caching;
    }

    public void setMaxLoadedJsps(int maxLoadedJsps) {
        this.maxLoadedJsps = maxLoadedJsps;
    }

    public void setJspIdleTimeout(int jspIdleTimeout) {
        this.jspIdleTimeout = jspIdleTimeout;
    }

    public void setTldCache(TldCache tldCache) {
        this.tldCache = tldCache;
    }

    public void setJspConfig(JspConfig jspConfig) {
        this.jspConfig = jspConfig;
    }

    public void setTagPluginManager(TagPluginManager tagPluginManager) {
        this.tagPluginManager = tagPluginManager;
    }
    
    
    
}
