/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package es.rickyepoderi.compiler.jsp;

import es.rickyepoderi.compiler.SimpleCompiler;
import java.io.ByteArrayOutputStream;
import java.io.FileNotFoundException;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import org.apache.jasper.JasperException;
import org.apache.jasper.compiler.Compiler;
import org.apache.jasper.compiler.ServletWriter;

/**
 * Class that extends the jasper 2 compiler in order to not write files in 
 * any working directory. Besides it completes the generateClass method using
 * the SimpleCompiler class.
 * 
 * @author ricky
 */
public class MemoryCompiler extends Compiler {

    /**
     * The byte array in which the class is written.
     */
    private ByteArrayOutputStream bos = null;
    
    /**
     * The generated Class to be recovered after the compilation of the JSP.
     */
    private Class generatedClass = null;
    
    /**
     * This method is private in the jasper Compiler class. Nevertheless it is
     * needed to be overrided in order to avoid files generated in the file system.
     * 
     * @param javaFileName The java file name to create
     * @return The servlet write in which the java source will be written
     * @throws FileNotFoundException Error if the file is not foind (N/A in this implementation)
     * @throws JasperException Some error
     */
    @Override
    protected ServletWriter setupContextWriter(String javaFileName) throws FileNotFoundException, JasperException {
        ServletWriter writer;
        // Setup the ServletWriter
        String javaEncoding = ctxt.getOptions().getJavaEncoding();
        OutputStreamWriter osw = null;
        try {
            bos = new ByteArrayOutputStream();
            osw = new OutputStreamWriter(bos, javaEncoding);
        } catch (UnsupportedEncodingException ex) {
            errDispatcher.jspError("jsp.error.needAlternateJavaEncoding", javaEncoding);
        }
        writer = new ServletWriter(new PrintWriter(osw));
        ctxt.setWriter(writer);
        return writer;
    }

    /**
     * The method that generates the class (from the java source) is implemented
     * using the SimpleCompiler.
     * @param smap 
     * @throws FileNotFoundException
     * @throws JasperException
     * @throws Exception 
     */
    @Override
    protected void generateClass(String[] smap) throws FileNotFoundException, JasperException, Exception {
        SimpleCompiler compiler = new SimpleCompiler(ctxt.getClassLoader(), ctxt.getOptions());
        //System.err.println(bos.toString());
        generatedClass = compiler.compile(ctxt.getServletPackageName() + "." + ctxt.getServletClassName(), bos.toString());
    }
    
    /**
     * Getter for the compiled class if the compilation run smoothly.
     * @return The class compiled
     */
    public Class getGeneratedClass() {
        return generatedClass;
    }
    
}